# coding: utf-8
# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from .create_autonomous_database_base import CreateAutonomousDatabaseBase
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CreateAutonomousDatabaseFromBackupTimestampDetails(CreateAutonomousDatabaseBase):
    """
    Details to create a point-in-time clone of an Oracle Autonomous Database by specifying a timestamp. Point-in-time clones use backups as the source of the data for the clone.
    """

    #: A constant which can be used with the clone_type property of a CreateAutonomousDatabaseFromBackupTimestampDetails.
    #: This constant has a value of "FULL"
    CLONE_TYPE_FULL = "FULL"

    #: A constant which can be used with the clone_type property of a CreateAutonomousDatabaseFromBackupTimestampDetails.
    #: This constant has a value of "METADATA"
    CLONE_TYPE_METADATA = "METADATA"

    def __init__(self, **kwargs):
        """
        Initializes a new CreateAutonomousDatabaseFromBackupTimestampDetails object with values from keyword arguments. The default value of the :py:attr:`~oci.database.models.CreateAutonomousDatabaseFromBackupTimestampDetails.source` attribute
        of this class is ``BACKUP_FROM_TIMESTAMP`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param compartment_id:
            The value to assign to the compartment_id property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type compartment_id: str

        :param db_name:
            The value to assign to the db_name property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type db_name: str

        :param cpu_core_count:
            The value to assign to the cpu_core_count property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type cpu_core_count: int

        :param ocpu_count:
            The value to assign to the ocpu_count property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type ocpu_count: float

        :param db_workload:
            The value to assign to the db_workload property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
            Allowed values for this property are: "OLTP", "DW", "AJD", "APEX"
        :type db_workload: str

        :param data_storage_size_in_tbs:
            The value to assign to the data_storage_size_in_tbs property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type data_storage_size_in_tbs: int

        :param data_storage_size_in_gbs:
            The value to assign to the data_storage_size_in_gbs property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type data_storage_size_in_gbs: int

        :param is_free_tier:
            The value to assign to the is_free_tier property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type is_free_tier: bool

        :param kms_key_id:
            The value to assign to the kms_key_id property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type kms_key_id: str

        :param vault_id:
            The value to assign to the vault_id property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type vault_id: str

        :param admin_password:
            The value to assign to the admin_password property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type admin_password: str

        :param display_name:
            The value to assign to the display_name property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type display_name: str

        :param license_model:
            The value to assign to the license_model property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
            Allowed values for this property are: "LICENSE_INCLUDED", "BRING_YOUR_OWN_LICENSE"
        :type license_model: str

        :param is_preview_version_with_service_terms_accepted:
            The value to assign to the is_preview_version_with_service_terms_accepted property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type is_preview_version_with_service_terms_accepted: bool

        :param is_auto_scaling_enabled:
            The value to assign to the is_auto_scaling_enabled property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type is_auto_scaling_enabled: bool

        :param is_dedicated:
            The value to assign to the is_dedicated property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type is_dedicated: bool

        :param autonomous_container_database_id:
            The value to assign to the autonomous_container_database_id property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type autonomous_container_database_id: str

        :param is_access_control_enabled:
            The value to assign to the is_access_control_enabled property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type is_access_control_enabled: bool

        :param whitelisted_ips:
            The value to assign to the whitelisted_ips property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type whitelisted_ips: list[str]

        :param are_primary_whitelisted_ips_used:
            The value to assign to the are_primary_whitelisted_ips_used property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type are_primary_whitelisted_ips_used: bool

        :param standby_whitelisted_ips:
            The value to assign to the standby_whitelisted_ips property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type standby_whitelisted_ips: list[str]

        :param is_data_guard_enabled:
            The value to assign to the is_data_guard_enabled property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type is_data_guard_enabled: bool

        :param subnet_id:
            The value to assign to the subnet_id property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type subnet_id: str

        :param nsg_ids:
            The value to assign to the nsg_ids property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type nsg_ids: list[str]

        :param private_endpoint_label:
            The value to assign to the private_endpoint_label property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type private_endpoint_label: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type freeform_tags: dict(str, str)

        :param defined_tags:
            The value to assign to the defined_tags property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type defined_tags: dict(str, dict(str, object))

        :param db_version:
            The value to assign to the db_version property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type db_version: str

        :param source:
            The value to assign to the source property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
            Allowed values for this property are: "NONE", "DATABASE", "BACKUP_FROM_ID", "BACKUP_FROM_TIMESTAMP", "CLONE_TO_REFRESHABLE", "CROSS_REGION_DATAGUARD"
        :type source: str

        :param customer_contacts:
            The value to assign to the customer_contacts property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type customer_contacts: list[oci.database.models.CustomerContact]

        :param is_mtls_connection_required:
            The value to assign to the is_mtls_connection_required property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type is_mtls_connection_required: bool

        :param autonomous_maintenance_schedule_type:
            The value to assign to the autonomous_maintenance_schedule_type property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
            Allowed values for this property are: "EARLY", "REGULAR"
        :type autonomous_maintenance_schedule_type: str

        :param autonomous_database_id:
            The value to assign to the autonomous_database_id property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type autonomous_database_id: str

        :param timestamp:
            The value to assign to the timestamp property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type timestamp: datetime

        :param clone_type:
            The value to assign to the clone_type property of this CreateAutonomousDatabaseFromBackupTimestampDetails.
            Allowed values for this property are: "FULL", "METADATA"
        :type clone_type: str

        """
        self.swagger_types = {
            'compartment_id': 'str',
            'db_name': 'str',
            'cpu_core_count': 'int',
            'ocpu_count': 'float',
            'db_workload': 'str',
            'data_storage_size_in_tbs': 'int',
            'data_storage_size_in_gbs': 'int',
            'is_free_tier': 'bool',
            'kms_key_id': 'str',
            'vault_id': 'str',
            'admin_password': 'str',
            'display_name': 'str',
            'license_model': 'str',
            'is_preview_version_with_service_terms_accepted': 'bool',
            'is_auto_scaling_enabled': 'bool',
            'is_dedicated': 'bool',
            'autonomous_container_database_id': 'str',
            'is_access_control_enabled': 'bool',
            'whitelisted_ips': 'list[str]',
            'are_primary_whitelisted_ips_used': 'bool',
            'standby_whitelisted_ips': 'list[str]',
            'is_data_guard_enabled': 'bool',
            'subnet_id': 'str',
            'nsg_ids': 'list[str]',
            'private_endpoint_label': 'str',
            'freeform_tags': 'dict(str, str)',
            'defined_tags': 'dict(str, dict(str, object))',
            'db_version': 'str',
            'source': 'str',
            'customer_contacts': 'list[CustomerContact]',
            'is_mtls_connection_required': 'bool',
            'autonomous_maintenance_schedule_type': 'str',
            'autonomous_database_id': 'str',
            'timestamp': 'datetime',
            'clone_type': 'str'
        }

        self.attribute_map = {
            'compartment_id': 'compartmentId',
            'db_name': 'dbName',
            'cpu_core_count': 'cpuCoreCount',
            'ocpu_count': 'ocpuCount',
            'db_workload': 'dbWorkload',
            'data_storage_size_in_tbs': 'dataStorageSizeInTBs',
            'data_storage_size_in_gbs': 'dataStorageSizeInGBs',
            'is_free_tier': 'isFreeTier',
            'kms_key_id': 'kmsKeyId',
            'vault_id': 'vaultId',
            'admin_password': 'adminPassword',
            'display_name': 'displayName',
            'license_model': 'licenseModel',
            'is_preview_version_with_service_terms_accepted': 'isPreviewVersionWithServiceTermsAccepted',
            'is_auto_scaling_enabled': 'isAutoScalingEnabled',
            'is_dedicated': 'isDedicated',
            'autonomous_container_database_id': 'autonomousContainerDatabaseId',
            'is_access_control_enabled': 'isAccessControlEnabled',
            'whitelisted_ips': 'whitelistedIps',
            'are_primary_whitelisted_ips_used': 'arePrimaryWhitelistedIpsUsed',
            'standby_whitelisted_ips': 'standbyWhitelistedIps',
            'is_data_guard_enabled': 'isDataGuardEnabled',
            'subnet_id': 'subnetId',
            'nsg_ids': 'nsgIds',
            'private_endpoint_label': 'privateEndpointLabel',
            'freeform_tags': 'freeformTags',
            'defined_tags': 'definedTags',
            'db_version': 'dbVersion',
            'source': 'source',
            'customer_contacts': 'customerContacts',
            'is_mtls_connection_required': 'isMtlsConnectionRequired',
            'autonomous_maintenance_schedule_type': 'autonomousMaintenanceScheduleType',
            'autonomous_database_id': 'autonomousDatabaseId',
            'timestamp': 'timestamp',
            'clone_type': 'cloneType'
        }

        self._compartment_id = None
        self._db_name = None
        self._cpu_core_count = None
        self._ocpu_count = None
        self._db_workload = None
        self._data_storage_size_in_tbs = None
        self._data_storage_size_in_gbs = None
        self._is_free_tier = None
        self._kms_key_id = None
        self._vault_id = None
        self._admin_password = None
        self._display_name = None
        self._license_model = None
        self._is_preview_version_with_service_terms_accepted = None
        self._is_auto_scaling_enabled = None
        self._is_dedicated = None
        self._autonomous_container_database_id = None
        self._is_access_control_enabled = None
        self._whitelisted_ips = None
        self._are_primary_whitelisted_ips_used = None
        self._standby_whitelisted_ips = None
        self._is_data_guard_enabled = None
        self._subnet_id = None
        self._nsg_ids = None
        self._private_endpoint_label = None
        self._freeform_tags = None
        self._defined_tags = None
        self._db_version = None
        self._source = None
        self._customer_contacts = None
        self._is_mtls_connection_required = None
        self._autonomous_maintenance_schedule_type = None
        self._autonomous_database_id = None
        self._timestamp = None
        self._clone_type = None
        self._source = 'BACKUP_FROM_TIMESTAMP'

    @property
    def autonomous_database_id(self):
        """
        **[Required]** Gets the autonomous_database_id of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        The `OCID`__ of the source Autonomous Database that you will clone to create a new Autonomous Database.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :return: The autonomous_database_id of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :rtype: str
        """
        return self._autonomous_database_id

    @autonomous_database_id.setter
    def autonomous_database_id(self, autonomous_database_id):
        """
        Sets the autonomous_database_id of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        The `OCID`__ of the source Autonomous Database that you will clone to create a new Autonomous Database.

        __ https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm


        :param autonomous_database_id: The autonomous_database_id of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type: str
        """
        self._autonomous_database_id = autonomous_database_id

    @property
    def timestamp(self):
        """
        **[Required]** Gets the timestamp of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        The timestamp specified for the point-in-time clone of the source Autonomous Database. The timestamp must be in the past.


        :return: The timestamp of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :rtype: datetime
        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp):
        """
        Sets the timestamp of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        The timestamp specified for the point-in-time clone of the source Autonomous Database. The timestamp must be in the past.


        :param timestamp: The timestamp of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type: datetime
        """
        self._timestamp = timestamp

    @property
    def clone_type(self):
        """
        **[Required]** Gets the clone_type of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        The Autonomous Database clone type.

        Allowed values for this property are: "FULL", "METADATA"


        :return: The clone_type of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :rtype: str
        """
        return self._clone_type

    @clone_type.setter
    def clone_type(self, clone_type):
        """
        Sets the clone_type of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        The Autonomous Database clone type.


        :param clone_type: The clone_type of this CreateAutonomousDatabaseFromBackupTimestampDetails.
        :type: str
        """
        allowed_values = ["FULL", "METADATA"]
        if not value_allowed_none_or_none_sentinel(clone_type, allowed_values):
            raise ValueError(
                "Invalid value for `clone_type`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._clone_type = clone_type

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class ReportSummary(object):
    """
    The model of a single report.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new ReportSummary object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param report_type:
            The value to assign to the report_type property of this ReportSummary.
        :type report_type: str

        :param date:
            The value to assign to the date property of this ReportSummary.
        :type date: datetime

        :param columns:
            The value to assign to the columns property of this ReportSummary.
        :type columns: list[str]

        :param content:
            The value to assign to the content property of this ReportSummary.
        :type content: str

        """
        self.swagger_types = {
            'report_type': 'str',
            'date': 'datetime',
            'columns': 'list[str]',
            'content': 'str'
        }

        self.attribute_map = {
            'report_type': 'reportType',
            'date': 'date',
            'columns': 'columns',
            'content': 'content'
        }

        self._report_type = None
        self._date = None
        self._columns = None
        self._content = None

    @property
    def report_type(self):
        """
        **[Required]** Gets the report_type of this ReportSummary.
        The type of report.


        :return: The report_type of this ReportSummary.
        :rtype: str
        """
        return self._report_type

    @report_type.setter
    def report_type(self, report_type):
        """
        Sets the report_type of this ReportSummary.
        The type of report.


        :param report_type: The report_type of this ReportSummary.
        :type: str
        """
        self._report_type = report_type

    @property
    def date(self):
        """
        **[Required]** Gets the date of this ReportSummary.
        The date of the report.


        :return: The date of this ReportSummary.
        :rtype: datetime
        """
        return self._date

    @date.setter
    def date(self, date):
        """
        Sets the date of this ReportSummary.
        The date of the report.


        :param date: The date of this ReportSummary.
        :type: datetime
        """
        self._date = date

    @property
    def columns(self):
        """
        **[Required]** Gets the columns of this ReportSummary.
        The columns in the report.


        :return: The columns of this ReportSummary.
        :rtype: list[str]
        """
        return self._columns

    @columns.setter
    def columns(self, columns):
        """
        Sets the columns of this ReportSummary.
        The columns in the report.


        :param columns: The columns of this ReportSummary.
        :type: list[str]
        """
        self._columns = columns

    @property
    def content(self):
        """
        **[Required]** Gets the content of this ReportSummary.
        The contents of the report in comma-separated values (CSV) file format.


        :return: The content of this ReportSummary.
        :rtype: str
        """
        return self._content

    @content.setter
    def content(self, content):
        """
        Sets the content of this ReportSummary.
        The contents of the report in comma-separated values (CSV) file format.


        :param content: The content of this ReportSummary.
        :type: str
        """
        self._content = content

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

"""
Helpers for loading and validating service configuration.

You can configure services by passing a config dict directly, or by loading
one from a configuration file.

The following keys are required::

    fingerprint
    key_file
    region
    tenancy
    user

Additionally, the following keys are optional::

    additional_user_agent (default "")
    log_requests (default False)
    pass_phrase (required if your key_file has a passphrase)

"""

from __future__ import absolute_import
import configparser
import os
import re
import logging
from oci._vendor import six

from .exceptions import ConfigFileNotFound, ProfileNotFound, InvalidConfig, InvalidKeyFilePath
from .auth import signers
from .util import AUTHENTICATION_TYPE_FIELD_NAME, get_authentication_type_from_config, DELEGATION_TOKEN_FILE_FIELD_NAME, DELEGATION_TOKEN_WITH_INSTANCE_PRINCIPAL_AUTHENTICATION_TYPE

__all__ = ["DEFAULT_CONFIG", "from_file", "validate_config"]

DEFAULT_CONFIG = {
    "log_requests": False,
    "additional_user_agent": "",
    "pass_phrase": None
}
DEFAULT_LOCATION = os.path.join('~', '.oci', 'config')
FALLBACK_DEFAULT_LOCATION = os.path.join('~', '.oraclebmc', 'config')
DEFAULT_PROFILE = "DEFAULT"
PATTERNS = {
    # Tenancy and user have the same shape
    "tenancy": re.compile("^([0-9a-zA-Z-_]+[.:])([0-9a-zA-Z-_]*[.:]){3,}([0-9a-zA-Z-_]+)$"),
    "user": re.compile("^([0-9a-zA-Z-_]+[.:])([0-9a-zA-Z-_]*[.:]){3,}([0-9a-zA-Z-_]+)$"),
    "fingerprint": re.compile("^([0-9a-f]{2}:){15}[0-9a-f]{2}$")
}
REQUIRED = {
    "user",
    "tenancy",
    "fingerprint",
    "key_file",
    "region"
}
REQUIRED_FALLBACKS = {
    "key_file": "key_content"
}
CONFIG_FILE_BLACKLISTED_KEYS = {
    "key_content"
}

CONFIG_FILE_PATH_ENV_VAR_NAME = "OCI_CONFIG_FILE"
REGION_ENV_VAR_NAME = "OCI_REGION"
REGION_KEY_NAME = "region"

logger = logging.getLogger(__name__)


def _validate_delegation_token_with_instance_principal(config):
    # At this point we know we have the required fields for this authentication type, so we won't check that again
    delegation_token_file_path = config.get(DELEGATION_TOKEN_FILE_FIELD_NAME)
    if delegation_token_file_path is None:
        raise InvalidConfig('ERROR: Please specify the location of the delegation_token_file in the config.')

    expanded_delegation_token_file_path = os.path.expanduser(delegation_token_file_path)

    if not os.path.isfile(expanded_delegation_token_file_path):
        raise InvalidConfig("Delegation token file not found at {}".format(expanded_delegation_token_file_path))


# Map the validator function for each type
# This can easily be extended to support other auth types
AUTH_TYPE_TO_VALIDATION_FUNCTION_MAP = {
    DELEGATION_TOKEN_WITH_INSTANCE_PRINCIPAL_AUTHENTICATION_TYPE: _validate_delegation_token_with_instance_principal
}


def from_file(file_location=DEFAULT_LOCATION, profile_name=DEFAULT_PROFILE):
    """Create a config dict from a file.

    :param file_location: Path to the config file.  Defaults to ~/.oci/config and with a fallback
                            to environment variable OCI_CONFIG_FILE, then ~/.oraclebmc/config.
    :param profile_name: The profile to load from the config file.  Defaults to "DEFAULT"
    :return: A config dict that can be used to create clients.
    """
    expanded_file_location = _get_config_path_with_fallback(file_location)

    parser = configparser.ConfigParser(interpolation=None)
    if not parser.read(expanded_file_location):
        raise ConfigFileNotFound("Could not find config file at {}, please follow the instructions in the link to setup the config file https://docs.cloud.oracle.com/en-us/iaas/Content/API/Concepts/sdkconfig.htm".format(expanded_file_location))

    if profile_name not in parser:
        raise ProfileNotFound("Profile '{}' not found in config file {}".format(profile_name, expanded_file_location))

    config = dict(DEFAULT_CONFIG)
    config.update(parser[profile_name])
    config["log_requests"] = _as_bool(config["log_requests"])

    for key in CONFIG_FILE_BLACKLISTED_KEYS:
        if key in config:
            raise ValueError("'{}' cannot be specified in a config file for security reasons. To use this key you must add it to the config programmatically.".format(key))

    invalid_key_file_path_checker(config, expanded_file_location, profile_name)

    return config


def validate_config(config, **kwargs):
    if 'signer' in kwargs:
        # InstancePrincipalsSecurityTokenSigner and  SecurityTokenSigner are
        # self-sufficient and do not need to read the normally-required keys
        # in the config
        if isinstance(kwargs['signer'], signers.InstancePrincipalsSecurityTokenSigner) or isinstance(kwargs['signer'], signers.SecurityTokenSigner):
            return

    if AUTHENTICATION_TYPE_FIELD_NAME in config:
        auth_type = get_authentication_type_from_config(config)
        validator_function = AUTH_TYPE_TO_VALIDATION_FUNCTION_MAP.get(auth_type)
        validator_function(config)
        return

    """Raises ValueError if required fields are missing or malformed."""
    errors = {}
    for required_key in REQUIRED:
        fallback_key = REQUIRED_FALLBACKS.get(required_key)
        if (required_key not in config or config[required_key] is None) and (fallback_key not in config or config[fallback_key] is None):
            # If region is not provided, check the env variable
            if required_key == REGION_KEY_NAME:
                logger.debug("Region not found in config, checking environment variable {}".format(REGION_ENV_VAR_NAME))
                region_from_env_var = os.environ.get(REGION_ENV_VAR_NAME)
                if region_from_env_var:
                    # If present, inject the region in config
                    logger.debug("Setting region from environment variable {}".format(REGION_ENV_VAR_NAME))
                    config[REGION_KEY_NAME] = region_from_env_var
                else:
                    errors[required_key] = "missing"
            else:
                errors[required_key] = "missing"

    for key, pattern in six.iteritems(PATTERNS):
        if key in errors:
            # key is missing, can't possibly match pattern
            continue
        if not pattern.match(config[key]):
            errors[key] = "malformed"
    if errors:
        raise InvalidConfig(errors)


def get_config_value_or_default(config, key):
    return config.get(key, DEFAULT_CONFIG.get(key))


def _as_bool(x):
    if x in [True, False]:
        return x
    if x.lower() in ["1", "yes", "true", "on"]:
        return True
    elif x.lower() in ["0", "no", "false", "off"]:
        return False
    else:
        raise ValueError("{!r} is not a valid alias for True/False".format(x))


def _raise_on_errors(errors):
    # report all errors at once
    if len(errors) == 1:
        raise ValueError("Error in config: {}".format(errors[0]))
    elif errors:
        raise ValueError("Found the following config errors: {!r}".format(errors))


def _get_config_path_with_fallback(file_location):
    expanded_file_location = os.path.expanduser(file_location)
    expanded_fallback_default_file_location = os.path.expanduser(FALLBACK_DEFAULT_LOCATION)

    if (file_location != DEFAULT_LOCATION) or (file_location == DEFAULT_LOCATION and os.path.isfile(expanded_file_location)):
        logger.debug("Config file found at {}".format(file_location))
        return expanded_file_location

    # If file location is not specified and the default file (~/.oci/config) does not exist
    # then try getting config file path from env var
    elif os.environ.get(CONFIG_FILE_PATH_ENV_VAR_NAME):
        logger.debug(
            "No file location specified and default file does not exist. Getting path info from the environment variable {}".format(
                CONFIG_FILE_PATH_ENV_VAR_NAME))
        expanded_file_location = os.path.expanduser(os.environ.get(CONFIG_FILE_PATH_ENV_VAR_NAME))
        return expanded_file_location

    # If we cannot determine the path from any other source and the fallback path (~/.oraclebmc/config) exists,
    # use that path
    elif os.path.isfile(expanded_fallback_default_file_location):
        expanded_file_location = expanded_fallback_default_file_location
        return expanded_file_location

    logger.debug("Config file found at {}".format(expanded_file_location))
    return expanded_file_location


def invalid_key_file_path_checker(config, expanded_file_location, profile_name):
    if 'key_file' in config:
        key_file_path = os.path.expanduser(config.get('key_file'))
        if not os.path.isfile(key_file_path):
            line_number = get_linenum_from_file(config.get('key_file'), expanded_file_location, profile_name)
            line_message = " at line {}".format(line_number) if line_number else ""
            message = "Config file {} is invalid: the key_file's value \'{}\'{} must be a valid file path.".format(expanded_file_location, key_file_path, line_message)
            raise InvalidKeyFilePath(message)


def get_linenum_from_file(key, filename, profile):
    with open(filename, 'r') as f:
        profile_found = False
        profile_token = "[" + profile + "]"
        for line_number, line in enumerate(f):
            line = line.strip()
            if len(line) >= 1:
                if line[0] == "#":
                    continue
            if profile_token in line:
                profile_found = True
            if profile_found and key in line:
                return line_number + 1
    return None



# coding: utf-8
# Modified Work: Copyright (c) 2018, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
# Original Work: Copyright (c) 2018 Character Encoding Detector contributors.  https://github.com/chardet

from .charsetprober import CharSetProber
from .enums import ProbingState

FREQ_CAT_NUM = 4

UDF = 0  # undefined
OTH = 1  # other
ASC = 2  # ascii capital letter
ASS = 3  # ascii small letter
ACV = 4  # accent capital vowel
ACO = 5  # accent capital other
ASV = 6  # accent small vowel
ASO = 7  # accent small other
CLASS_NUM = 8  # total classes

Latin1_CharToClass = (
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 00 - 07
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 08 - 0F
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 10 - 17
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 18 - 1F
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 20 - 27
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 28 - 2F
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 30 - 37
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 38 - 3F
    OTH, ASC, ASC, ASC, ASC, ASC, ASC, ASC,   # 40 - 47
    ASC, ASC, ASC, ASC, ASC, ASC, ASC, ASC,   # 48 - 4F
    ASC, ASC, ASC, ASC, ASC, ASC, ASC, ASC,   # 50 - 57
    ASC, ASC, ASC, OTH, OTH, OTH, OTH, OTH,   # 58 - 5F
    OTH, ASS, ASS, ASS, ASS, ASS, ASS, ASS,   # 60 - 67
    ASS, ASS, ASS, ASS, ASS, ASS, ASS, ASS,   # 68 - 6F
    ASS, ASS, ASS, ASS, ASS, ASS, ASS, ASS,   # 70 - 77
    ASS, ASS, ASS, OTH, OTH, OTH, OTH, OTH,   # 78 - 7F
    OTH, UDF, OTH, ASO, OTH, OTH, OTH, OTH,   # 80 - 87
    OTH, OTH, ACO, OTH, ACO, UDF, ACO, UDF,   # 88 - 8F
    UDF, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # 90 - 97
    OTH, OTH, ASO, OTH, ASO, UDF, ASO, ACO,   # 98 - 9F
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # A0 - A7
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # A8 - AF
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # B0 - B7
    OTH, OTH, OTH, OTH, OTH, OTH, OTH, OTH,   # B8 - BF
    ACV, ACV, ACV, ACV, ACV, ACV, ACO, ACO,   # C0 - C7
    ACV, ACV, ACV, ACV, ACV, ACV, ACV, ACV,   # C8 - CF
    ACO, ACO, ACV, ACV, ACV, ACV, ACV, OTH,   # D0 - D7
    ACV, ACV, ACV, ACV, ACV, ACO, ACO, ACO,   # D8 - DF
    ASV, ASV, ASV, ASV, ASV, ASV, ASO, ASO,   # E0 - E7
    ASV, ASV, ASV, ASV, ASV, ASV, ASV, ASV,   # E8 - EF
    ASO, ASO, ASV, ASV, ASV, ASV, ASV, OTH,   # F0 - F7
    ASV, ASV, ASV, ASV, ASV, ASO, ASO, ASO,   # F8 - FF
)

# 0 : illegal
# 1 : very unlikely
# 2 : normal
# 3 : very likely
Latin1ClassModel = (
# UDF OTH ASC ASS ACV ACO ASV ASO
    0,  0,  0,  0,  0,  0,  0,  0,  # UDF
    0,  3,  3,  3,  3,  3,  3,  3,  # OTH
    0,  3,  3,  3,  3,  3,  3,  3,  # ASC
    0,  3,  3,  3,  1,  1,  3,  3,  # ASS
    0,  3,  3,  3,  1,  2,  1,  2,  # ACV
    0,  3,  3,  3,  3,  3,  3,  3,  # ACO
    0,  3,  1,  3,  1,  1,  1,  3,  # ASV
    0,  3,  1,  3,  1,  1,  3,  3,  # ASO
)


class Latin1Prober(CharSetProber):
    def __init__(self):
        super(Latin1Prober, self).__init__()
        self._last_char_class = None
        self._freq_counter = None
        self.reset()

    def reset(self):
        self._last_char_class = OTH
        self._freq_counter = [0] * FREQ_CAT_NUM
        CharSetProber.reset(self)

    @property
    def charset_name(self):
        return "ISO-8859-1"

    @property
    def language(self):
        return ""

    def feed(self, byte_str):
        byte_str = self.filter_with_english_letters(byte_str)
        for c in byte_str:
            char_class = Latin1_CharToClass[c]
            freq = Latin1ClassModel[(self._last_char_class * CLASS_NUM)
                                    + char_class]
            if freq == 0:
                self._state = ProbingState.NOT_ME
                break
            self._freq_counter[freq] += 1
            self._last_char_class = char_class

        return self.state

    def get_confidence(self):
        if self.state == ProbingState.NOT_ME:
            return 0.01

        total = sum(self._freq_counter)
        if total < 0.01:
            confidence = 0.0
        else:
            confidence = ((self._freq_counter[3] - self._freq_counter[1] * 20.0)
                          / total)
        if confidence < 0.0:
            confidence = 0.0
        # lower the confidence of latin1 so that other more accurate
        # detector can take priority.
        confidence = confidence * 0.73
        return confidence



# coding: utf-8
# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class UpdateDataAssetDetails(object):
    """
    Properties used in data asset update operations.
    """

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "ORACLE_DATA_ASSET"
    MODEL_TYPE_ORACLE_DATA_ASSET = "ORACLE_DATA_ASSET"

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "ORACLE_OBJECT_STORAGE_DATA_ASSET"
    MODEL_TYPE_ORACLE_OBJECT_STORAGE_DATA_ASSET = "ORACLE_OBJECT_STORAGE_DATA_ASSET"

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "ORACLE_ATP_DATA_ASSET"
    MODEL_TYPE_ORACLE_ATP_DATA_ASSET = "ORACLE_ATP_DATA_ASSET"

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "ORACLE_ADWC_DATA_ASSET"
    MODEL_TYPE_ORACLE_ADWC_DATA_ASSET = "ORACLE_ADWC_DATA_ASSET"

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "MYSQL_DATA_ASSET"
    MODEL_TYPE_MYSQL_DATA_ASSET = "MYSQL_DATA_ASSET"

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "GENERIC_JDBC_DATA_ASSET"
    MODEL_TYPE_GENERIC_JDBC_DATA_ASSET = "GENERIC_JDBC_DATA_ASSET"

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "FUSION_APP_DATA_ASSET"
    MODEL_TYPE_FUSION_APP_DATA_ASSET = "FUSION_APP_DATA_ASSET"

    #: A constant which can be used with the model_type property of a UpdateDataAssetDetails.
    #: This constant has a value of "AMAZON_S3_DATA_ASSET"
    MODEL_TYPE_AMAZON_S3_DATA_ASSET = "AMAZON_S3_DATA_ASSET"

    def __init__(self, **kwargs):
        """
        Initializes a new UpdateDataAssetDetails object with values from keyword arguments. This class has the following subclasses and if you are using this class as input
        to a service operations then you should favor using a subclass over the base class:

        * :class:`~oci.data_integration.models.UpdateDataAssetFromAtp`
        * :class:`~oci.data_integration.models.UpdateDataAssetFromAdwc`
        * :class:`~oci.data_integration.models.UpdateDataAssetFromJdbc`
        * :class:`~oci.data_integration.models.UpdateDataAssetFromObjectStorage`
        * :class:`~oci.data_integration.models.UpdateDataAssetFromAmazonS3`
        * :class:`~oci.data_integration.models.UpdateDataAssetFromFusionApp`
        * :class:`~oci.data_integration.models.UpdateDataAssetFromMySQL`
        * :class:`~oci.data_integration.models.UpdateDataAssetFromOracle`

        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param model_type:
            The value to assign to the model_type property of this UpdateDataAssetDetails.
            Allowed values for this property are: "ORACLE_DATA_ASSET", "ORACLE_OBJECT_STORAGE_DATA_ASSET", "ORACLE_ATP_DATA_ASSET", "ORACLE_ADWC_DATA_ASSET", "MYSQL_DATA_ASSET", "GENERIC_JDBC_DATA_ASSET", "FUSION_APP_DATA_ASSET", "AMAZON_S3_DATA_ASSET"
        :type model_type: str

        :param key:
            The value to assign to the key property of this UpdateDataAssetDetails.
        :type key: str

        :param model_version:
            The value to assign to the model_version property of this UpdateDataAssetDetails.
        :type model_version: str

        :param name:
            The value to assign to the name property of this UpdateDataAssetDetails.
        :type name: str

        :param description:
            The value to assign to the description property of this UpdateDataAssetDetails.
        :type description: str

        :param object_status:
            The value to assign to the object_status property of this UpdateDataAssetDetails.
        :type object_status: int

        :param object_version:
            The value to assign to the object_version property of this UpdateDataAssetDetails.
        :type object_version: int

        :param identifier:
            The value to assign to the identifier property of this UpdateDataAssetDetails.
        :type identifier: str

        :param external_key:
            The value to assign to the external_key property of this UpdateDataAssetDetails.
        :type external_key: str

        :param asset_properties:
            The value to assign to the asset_properties property of this UpdateDataAssetDetails.
        :type asset_properties: dict(str, str)

        :param registry_metadata:
            The value to assign to the registry_metadata property of this UpdateDataAssetDetails.
        :type registry_metadata: oci.data_integration.models.RegistryMetadata

        """
        self.swagger_types = {
            'model_type': 'str',
            'key': 'str',
            'model_version': 'str',
            'name': 'str',
            'description': 'str',
            'object_status': 'int',
            'object_version': 'int',
            'identifier': 'str',
            'external_key': 'str',
            'asset_properties': 'dict(str, str)',
            'registry_metadata': 'RegistryMetadata'
        }

        self.attribute_map = {
            'model_type': 'modelType',
            'key': 'key',
            'model_version': 'modelVersion',
            'name': 'name',
            'description': 'description',
            'object_status': 'objectStatus',
            'object_version': 'objectVersion',
            'identifier': 'identifier',
            'external_key': 'externalKey',
            'asset_properties': 'assetProperties',
            'registry_metadata': 'registryMetadata'
        }

        self._model_type = None
        self._key = None
        self._model_version = None
        self._name = None
        self._description = None
        self._object_status = None
        self._object_version = None
        self._identifier = None
        self._external_key = None
        self._asset_properties = None
        self._registry_metadata = None

    @staticmethod
    def get_subtype(object_dictionary):
        """
        Given the hash representation of a subtype of this class,
        use the info in the hash to return the class of the subtype.
        """
        type = object_dictionary['modelType']

        if type == 'ORACLE_ATP_DATA_ASSET':
            return 'UpdateDataAssetFromAtp'

        if type == 'ORACLE_ADWC_DATA_ASSET':
            return 'UpdateDataAssetFromAdwc'

        if type == 'GENERIC_JDBC_DATA_ASSET':
            return 'UpdateDataAssetFromJdbc'

        if type == 'ORACLE_OBJECT_STORAGE_DATA_ASSET':
            return 'UpdateDataAssetFromObjectStorage'

        if type == 'AMAZON_S3_DATA_ASSET':
            return 'UpdateDataAssetFromAmazonS3'

        if type == 'FUSION_APP_DATA_ASSET':
            return 'UpdateDataAssetFromFusionApp'

        if type == 'MYSQL_DATA_ASSET':
            return 'UpdateDataAssetFromMySQL'

        if type == 'ORACLE_DATA_ASSET':
            return 'UpdateDataAssetFromOracle'
        else:
            return 'UpdateDataAssetDetails'

    @property
    def model_type(self):
        """
        **[Required]** Gets the model_type of this UpdateDataAssetDetails.
        The type of the data asset.

        Allowed values for this property are: "ORACLE_DATA_ASSET", "ORACLE_OBJECT_STORAGE_DATA_ASSET", "ORACLE_ATP_DATA_ASSET", "ORACLE_ADWC_DATA_ASSET", "MYSQL_DATA_ASSET", "GENERIC_JDBC_DATA_ASSET", "FUSION_APP_DATA_ASSET", "AMAZON_S3_DATA_ASSET"


        :return: The model_type of this UpdateDataAssetDetails.
        :rtype: str
        """
        return self._model_type

    @model_type.setter
    def model_type(self, model_type):
        """
        Sets the model_type of this UpdateDataAssetDetails.
        The type of the data asset.


        :param model_type: The model_type of this UpdateDataAssetDetails.
        :type: str
        """
        allowed_values = ["ORACLE_DATA_ASSET", "ORACLE_OBJECT_STORAGE_DATA_ASSET", "ORACLE_ATP_DATA_ASSET", "ORACLE_ADWC_DATA_ASSET", "MYSQL_DATA_ASSET", "GENERIC_JDBC_DATA_ASSET", "FUSION_APP_DATA_ASSET", "AMAZON_S3_DATA_ASSET"]
        if not value_allowed_none_or_none_sentinel(model_type, allowed_values):
            raise ValueError(
                "Invalid value for `model_type`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._model_type = model_type

    @property
    def key(self):
        """
        **[Required]** Gets the key of this UpdateDataAssetDetails.
        Generated key that can be used in API calls to identify data asset.


        :return: The key of this UpdateDataAssetDetails.
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """
        Sets the key of this UpdateDataAssetDetails.
        Generated key that can be used in API calls to identify data asset.


        :param key: The key of this UpdateDataAssetDetails.
        :type: str
        """
        self._key = key

    @property
    def model_version(self):
        """
        Gets the model_version of this UpdateDataAssetDetails.
        The model version of an object.


        :return: The model_version of this UpdateDataAssetDetails.
        :rtype: str
        """
        return self._model_version

    @model_version.setter
    def model_version(self, model_version):
        """
        Sets the model_version of this UpdateDataAssetDetails.
        The model version of an object.


        :param model_version: The model_version of this UpdateDataAssetDetails.
        :type: str
        """
        self._model_version = model_version

    @property
    def name(self):
        """
        Gets the name of this UpdateDataAssetDetails.
        Free form text without any restriction on permitted characters. Name can have letters, numbers, and special characters. The value is editable and is restricted to 1000 characters.


        :return: The name of this UpdateDataAssetDetails.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this UpdateDataAssetDetails.
        Free form text without any restriction on permitted characters. Name can have letters, numbers, and special characters. The value is editable and is restricted to 1000 characters.


        :param name: The name of this UpdateDataAssetDetails.
        :type: str
        """
        self._name = name

    @property
    def description(self):
        """
        Gets the description of this UpdateDataAssetDetails.
        The user-defined description of the data asset.


        :return: The description of this UpdateDataAssetDetails.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Sets the description of this UpdateDataAssetDetails.
        The user-defined description of the data asset.


        :param description: The description of this UpdateDataAssetDetails.
        :type: str
        """
        self._description = description

    @property
    def object_status(self):
        """
        Gets the object_status of this UpdateDataAssetDetails.
        The status of an object that can be set to value 1 for shallow references across objects, other values reserved.


        :return: The object_status of this UpdateDataAssetDetails.
        :rtype: int
        """
        return self._object_status

    @object_status.setter
    def object_status(self, object_status):
        """
        Sets the object_status of this UpdateDataAssetDetails.
        The status of an object that can be set to value 1 for shallow references across objects, other values reserved.


        :param object_status: The object_status of this UpdateDataAssetDetails.
        :type: int
        """
        self._object_status = object_status

    @property
    def object_version(self):
        """
        **[Required]** Gets the object_version of this UpdateDataAssetDetails.
        The version of the object that is used to track changes in the object instance.


        :return: The object_version of this UpdateDataAssetDetails.
        :rtype: int
        """
        return self._object_version

    @object_version.setter
    def object_version(self, object_version):
        """
        Sets the object_version of this UpdateDataAssetDetails.
        The version of the object that is used to track changes in the object instance.


        :param object_version: The object_version of this UpdateDataAssetDetails.
        :type: int
        """
        self._object_version = object_version

    @property
    def identifier(self):
        """
        Gets the identifier of this UpdateDataAssetDetails.
        Value can only contain upper case letters, underscore, and numbers. It should begin with upper case letter or underscore. The value can be modified.


        :return: The identifier of this UpdateDataAssetDetails.
        :rtype: str
        """
        return self._identifier

    @identifier.setter
    def identifier(self, identifier):
        """
        Sets the identifier of this UpdateDataAssetDetails.
        Value can only contain upper case letters, underscore, and numbers. It should begin with upper case letter or underscore. The value can be modified.


        :param identifier: The identifier of this UpdateDataAssetDetails.
        :type: str
        """
        self._identifier = identifier

    @property
    def external_key(self):
        """
        Gets the external_key of this UpdateDataAssetDetails.
        The external key for the object.


        :return: The external_key of this UpdateDataAssetDetails.
        :rtype: str
        """
        return self._external_key

    @external_key.setter
    def external_key(self, external_key):
        """
        Sets the external_key of this UpdateDataAssetDetails.
        The external key for the object.


        :param external_key: The external_key of this UpdateDataAssetDetails.
        :type: str
        """
        self._external_key = external_key

    @property
    def asset_properties(self):
        """
        Gets the asset_properties of this UpdateDataAssetDetails.
        Additional properties for the data asset.


        :return: The asset_properties of this UpdateDataAssetDetails.
        :rtype: dict(str, str)
        """
        return self._asset_properties

    @asset_properties.setter
    def asset_properties(self, asset_properties):
        """
        Sets the asset_properties of this UpdateDataAssetDetails.
        Additional properties for the data asset.


        :param asset_properties: The asset_properties of this UpdateDataAssetDetails.
        :type: dict(str, str)
        """
        self._asset_properties = asset_properties

    @property
    def registry_metadata(self):
        """
        Gets the registry_metadata of this UpdateDataAssetDetails.

        :return: The registry_metadata of this UpdateDataAssetDetails.
        :rtype: oci.data_integration.models.RegistryMetadata
        """
        return self._registry_metadata

    @registry_metadata.setter
    def registry_metadata(self, registry_metadata):
        """
        Sets the registry_metadata of this UpdateDataAssetDetails.

        :param registry_metadata: The registry_metadata of this UpdateDataAssetDetails.
        :type: oci.data_integration.models.RegistryMetadata
        """
        self._registry_metadata = registry_metadata

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class CreatePublicIpDetails(object):
    """
    CreatePublicIpDetails model.
    """

    #: A constant which can be used with the lifetime property of a CreatePublicIpDetails.
    #: This constant has a value of "EPHEMERAL"
    LIFETIME_EPHEMERAL = "EPHEMERAL"

    #: A constant which can be used with the lifetime property of a CreatePublicIpDetails.
    #: This constant has a value of "RESERVED"
    LIFETIME_RESERVED = "RESERVED"

    def __init__(self, **kwargs):
        """
        Initializes a new CreatePublicIpDetails object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param compartment_id:
            The value to assign to the compartment_id property of this CreatePublicIpDetails.
        :type compartment_id: str

        :param defined_tags:
            The value to assign to the defined_tags property of this CreatePublicIpDetails.
        :type defined_tags: dict(str, dict(str, object))

        :param display_name:
            The value to assign to the display_name property of this CreatePublicIpDetails.
        :type display_name: str

        :param freeform_tags:
            The value to assign to the freeform_tags property of this CreatePublicIpDetails.
        :type freeform_tags: dict(str, str)

        :param lifetime:
            The value to assign to the lifetime property of this CreatePublicIpDetails.
            Allowed values for this property are: "EPHEMERAL", "RESERVED"
        :type lifetime: str

        :param private_ip_id:
            The value to assign to the private_ip_id property of this CreatePublicIpDetails.
        :type private_ip_id: str

        :param public_ip_pool_id:
            The value to assign to the public_ip_pool_id property of this CreatePublicIpDetails.
        :type public_ip_pool_id: str

        """
        self.swagger_types = {
            'compartment_id': 'str',
            'defined_tags': 'dict(str, dict(str, object))',
            'display_name': 'str',
            'freeform_tags': 'dict(str, str)',
            'lifetime': 'str',
            'private_ip_id': 'str',
            'public_ip_pool_id': 'str'
        }

        self.attribute_map = {
            'compartment_id': 'compartmentId',
            'defined_tags': 'definedTags',
            'display_name': 'displayName',
            'freeform_tags': 'freeformTags',
            'lifetime': 'lifetime',
            'private_ip_id': 'privateIpId',
            'public_ip_pool_id': 'publicIpPoolId'
        }

        self._compartment_id = None
        self._defined_tags = None
        self._display_name = None
        self._freeform_tags = None
        self._lifetime = None
        self._private_ip_id = None
        self._public_ip_pool_id = None

    @property
    def compartment_id(self):
        """
        **[Required]** Gets the compartment_id of this CreatePublicIpDetails.
        The OCID of the compartment to contain the public IP. For ephemeral public IPs,
        you must set this to the private IP's compartment OCID.


        :return: The compartment_id of this CreatePublicIpDetails.
        :rtype: str
        """
        return self._compartment_id

    @compartment_id.setter
    def compartment_id(self, compartment_id):
        """
        Sets the compartment_id of this CreatePublicIpDetails.
        The OCID of the compartment to contain the public IP. For ephemeral public IPs,
        you must set this to the private IP's compartment OCID.


        :param compartment_id: The compartment_id of this CreatePublicIpDetails.
        :type: str
        """
        self._compartment_id = compartment_id

    @property
    def defined_tags(self):
        """
        Gets the defined_tags of this CreatePublicIpDetails.
        Defined tags for this resource. Each key is predefined and scoped to a
        namespace. For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :return: The defined_tags of this CreatePublicIpDetails.
        :rtype: dict(str, dict(str, object))
        """
        return self._defined_tags

    @defined_tags.setter
    def defined_tags(self, defined_tags):
        """
        Sets the defined_tags of this CreatePublicIpDetails.
        Defined tags for this resource. Each key is predefined and scoped to a
        namespace. For more information, see `Resource Tags`__.

        Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :param defined_tags: The defined_tags of this CreatePublicIpDetails.
        :type: dict(str, dict(str, object))
        """
        self._defined_tags = defined_tags

    @property
    def display_name(self):
        """
        Gets the display_name of this CreatePublicIpDetails.
        A user-friendly name. Does not have to be unique, and it's changeable. Avoid
        entering confidential information.


        :return: The display_name of this CreatePublicIpDetails.
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """
        Sets the display_name of this CreatePublicIpDetails.
        A user-friendly name. Does not have to be unique, and it's changeable. Avoid
        entering confidential information.


        :param display_name: The display_name of this CreatePublicIpDetails.
        :type: str
        """
        self._display_name = display_name

    @property
    def freeform_tags(self):
        """
        Gets the freeform_tags of this CreatePublicIpDetails.
        Free-form tags for this resource. Each tag is a simple key-value pair with no
        predefined name, type, or namespace. For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :return: The freeform_tags of this CreatePublicIpDetails.
        :rtype: dict(str, str)
        """
        return self._freeform_tags

    @freeform_tags.setter
    def freeform_tags(self, freeform_tags):
        """
        Sets the freeform_tags of this CreatePublicIpDetails.
        Free-form tags for this resource. Each tag is a simple key-value pair with no
        predefined name, type, or namespace. For more information, see `Resource Tags`__.

        Example: `{\"Department\": \"Finance\"}`

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/resourcetags.htm


        :param freeform_tags: The freeform_tags of this CreatePublicIpDetails.
        :type: dict(str, str)
        """
        self._freeform_tags = freeform_tags

    @property
    def lifetime(self):
        """
        **[Required]** Gets the lifetime of this CreatePublicIpDetails.
        Defines when the public IP is deleted and released back to the Oracle Cloud
        Infrastructure public IP pool. For more information, see
        `Public IP Addresses`__.

        __ https://docs.cloud.oracle.com/iaas/Content/Network/Tasks/managingpublicIPs.htm

        Allowed values for this property are: "EPHEMERAL", "RESERVED"


        :return: The lifetime of this CreatePublicIpDetails.
        :rtype: str
        """
        return self._lifetime

    @lifetime.setter
    def lifetime(self, lifetime):
        """
        Sets the lifetime of this CreatePublicIpDetails.
        Defines when the public IP is deleted and released back to the Oracle Cloud
        Infrastructure public IP pool. For more information, see
        `Public IP Addresses`__.

        __ https://docs.cloud.oracle.com/iaas/Content/Network/Tasks/managingpublicIPs.htm


        :param lifetime: The lifetime of this CreatePublicIpDetails.
        :type: str
        """
        allowed_values = ["EPHEMERAL", "RESERVED"]
        if not value_allowed_none_or_none_sentinel(lifetime, allowed_values):
            raise ValueError(
                "Invalid value for `lifetime`, must be None or one of {0}"
                .format(allowed_values)
            )
        self._lifetime = lifetime

    @property
    def private_ip_id(self):
        """
        Gets the private_ip_id of this CreatePublicIpDetails.
        The OCID of the private IP to assign the public IP to.

        Required for an ephemeral public IP because it must always be assigned to a private IP
        (specifically a *primary* private IP).

        Optional for a reserved public IP. If you don't provide it, the public IP is created but not
        assigned to a private IP. You can later assign the public IP with
        :func:`update_public_ip`.


        :return: The private_ip_id of this CreatePublicIpDetails.
        :rtype: str
        """
        return self._private_ip_id

    @private_ip_id.setter
    def private_ip_id(self, private_ip_id):
        """
        Sets the private_ip_id of this CreatePublicIpDetails.
        The OCID of the private IP to assign the public IP to.

        Required for an ephemeral public IP because it must always be assigned to a private IP
        (specifically a *primary* private IP).

        Optional for a reserved public IP. If you don't provide it, the public IP is created but not
        assigned to a private IP. You can later assign the public IP with
        :func:`update_public_ip`.


        :param private_ip_id: The private_ip_id of this CreatePublicIpDetails.
        :type: str
        """
        self._private_ip_id = private_ip_id

    @property
    def public_ip_pool_id(self):
        """
        Gets the public_ip_pool_id of this CreatePublicIpDetails.
        The `OCID`__ of the public IP pool.

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm


        :return: The public_ip_pool_id of this CreatePublicIpDetails.
        :rtype: str
        """
        return self._public_ip_pool_id

    @public_ip_pool_id.setter
    def public_ip_pool_id(self, public_ip_pool_id):
        """
        Sets the public_ip_pool_id of this CreatePublicIpDetails.
        The `OCID`__ of the public IP pool.

        __ https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm


        :param public_ip_pool_id: The public_ip_pool_id of this CreatePublicIpDetails.
        :type: str
        """
        self._public_ip_pool_id = public_ip_pool_id

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

from .connector_attribute import ConnectorAttribute
from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class ExternalStorage(ConnectorAttribute):
    """
    BICC Connector Attribute.Object Storage as External storage where the BICC extracted files are written
    """

    def __init__(self, **kwargs):
        """
        Initializes a new ExternalStorage object with values from keyword arguments. The default value of the :py:attr:`~oci.data_integration.models.ExternalStorage.model_type` attribute
        of this class is ``EXTERNAL_STORAGE`` and it should not be changed.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param model_type:
            The value to assign to the model_type property of this ExternalStorage.
            Allowed values for this property are: "EXTERNAL_STORAGE", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type model_type: str

        :param storage_id:
            The value to assign to the storage_id property of this ExternalStorage.
        :type storage_id: str

        :param storage_name:
            The value to assign to the storage_name property of this ExternalStorage.
        :type storage_name: str

        :param host:
            The value to assign to the host property of this ExternalStorage.
        :type host: str

        :param tenancy_id:
            The value to assign to the tenancy_id property of this ExternalStorage.
        :type tenancy_id: str

        :param namespace:
            The value to assign to the namespace property of this ExternalStorage.
        :type namespace: str

        :param bucket:
            The value to assign to the bucket property of this ExternalStorage.
        :type bucket: str

        """
        self.swagger_types = {
            'model_type': 'str',
            'storage_id': 'str',
            'storage_name': 'str',
            'host': 'str',
            'tenancy_id': 'str',
            'namespace': 'str',
            'bucket': 'str'
        }

        self.attribute_map = {
            'model_type': 'modelType',
            'storage_id': 'storageId',
            'storage_name': 'storageName',
            'host': 'host',
            'tenancy_id': 'tenancyId',
            'namespace': 'namespace',
            'bucket': 'bucket'
        }

        self._model_type = None
        self._storage_id = None
        self._storage_name = None
        self._host = None
        self._tenancy_id = None
        self._namespace = None
        self._bucket = None
        self._model_type = 'EXTERNAL_STORAGE'

    @property
    def storage_id(self):
        """
        Gets the storage_id of this ExternalStorage.
        Id of the external stoarge configured in BICC console. Usually its numeric.


        :return: The storage_id of this ExternalStorage.
        :rtype: str
        """
        return self._storage_id

    @storage_id.setter
    def storage_id(self, storage_id):
        """
        Sets the storage_id of this ExternalStorage.
        Id of the external stoarge configured in BICC console. Usually its numeric.


        :param storage_id: The storage_id of this ExternalStorage.
        :type: str
        """
        self._storage_id = storage_id

    @property
    def storage_name(self):
        """
        Gets the storage_name of this ExternalStorage.
        Name of the external storage configured in BICC console


        :return: The storage_name of this ExternalStorage.
        :rtype: str
        """
        return self._storage_name

    @storage_name.setter
    def storage_name(self, storage_name):
        """
        Sets the storage_name of this ExternalStorage.
        Name of the external storage configured in BICC console


        :param storage_name: The storage_name of this ExternalStorage.
        :type: str
        """
        self._storage_name = storage_name

    @property
    def host(self):
        """
        Gets the host of this ExternalStorage.
        Object Storage host Url. DO not give http/https.


        :return: The host of this ExternalStorage.
        :rtype: str
        """
        return self._host

    @host.setter
    def host(self, host):
        """
        Sets the host of this ExternalStorage.
        Object Storage host Url. DO not give http/https.


        :param host: The host of this ExternalStorage.
        :type: str
        """
        self._host = host

    @property
    def tenancy_id(self):
        """
        Gets the tenancy_id of this ExternalStorage.
        Tenancy OCID for the OOS bucket


        :return: The tenancy_id of this ExternalStorage.
        :rtype: str
        """
        return self._tenancy_id

    @tenancy_id.setter
    def tenancy_id(self, tenancy_id):
        """
        Sets the tenancy_id of this ExternalStorage.
        Tenancy OCID for the OOS bucket


        :param tenancy_id: The tenancy_id of this ExternalStorage.
        :type: str
        """
        self._tenancy_id = tenancy_id

    @property
    def namespace(self):
        """
        Gets the namespace of this ExternalStorage.
        Namespace for the OOS bucket


        :return: The namespace of this ExternalStorage.
        :rtype: str
        """
        return self._namespace

    @namespace.setter
    def namespace(self, namespace):
        """
        Sets the namespace of this ExternalStorage.
        Namespace for the OOS bucket


        :param namespace: The namespace of this ExternalStorage.
        :type: str
        """
        self._namespace = namespace

    @property
    def bucket(self):
        """
        Gets the bucket of this ExternalStorage.
        Bucket Name where BICC extracts stores the files


        :return: The bucket of this ExternalStorage.
        :rtype: str
        """
        return self._bucket

    @bucket.setter
    def bucket(self, bucket):
        """
        Sets the bucket of this ExternalStorage.
        Bucket Name where BICC extracts stores the files


        :param bucket: The bucket of this ExternalStorage.
        :type: str
        """
        self._bucket = bucket

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
# Copyright (c) 2016, 2021, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class OperatorSummary(object):
    """
    Summary of Operator
    """

    #: A constant which can be used with the filter_type property of a OperatorSummary.
    #: This constant has a value of "CONDITION"
    FILTER_TYPE_CONDITION = "CONDITION"

    #: A constant which can be used with the filter_type property of a OperatorSummary.
    #: This constant has a value of "CONFIG"
    FILTER_TYPE_CONFIG = "CONFIG"

    def __init__(self, **kwargs):
        """
        Initializes a new OperatorSummary object with values from keyword arguments.
        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param name:
            The value to assign to the name property of this OperatorSummary.
        :type name: str

        :param display_name:
            The value to assign to the display_name property of this OperatorSummary.
        :type display_name: str

        :param datatype:
            The value to assign to the datatype property of this OperatorSummary.
        :type datatype: str

        :param managed_listtype:
            The value to assign to the managed_listtype property of this OperatorSummary.
        :type managed_listtype: str

        :param filter_type:
            The value to assign to the filter_type property of this OperatorSummary.
            Allowed values for this property are: "CONDITION", "CONFIG", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type filter_type: str

        :param operators:
            The value to assign to the operators property of this OperatorSummary.
        :type operators: list[oci.cloud_guard.models.ConditionOperator]

        :param multi_list_types:
            The value to assign to the multi_list_types property of this OperatorSummary.
        :type multi_list_types: list[str]

        """
        self.swagger_types = {
            'name': 'str',
            'display_name': 'str',
            'datatype': 'str',
            'managed_listtype': 'str',
            'filter_type': 'str',
            'operators': 'list[ConditionOperator]',
            'multi_list_types': 'list[str]'
        }

        self.attribute_map = {
            'name': 'name',
            'display_name': 'displayName',
            'datatype': 'datatype',
            'managed_listtype': 'managedListtype',
            'filter_type': 'filterType',
            'operators': 'operators',
            'multi_list_types': 'multiListTypes'
        }

        self._name = None
        self._display_name = None
        self._datatype = None
        self._managed_listtype = None
        self._filter_type = None
        self._operators = None
        self._multi_list_types = None

    @property
    def name(self):
        """
        **[Required]** Gets the name of this OperatorSummary.
        name of the operand


        :return: The name of this OperatorSummary.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this OperatorSummary.
        name of the operand


        :param name: The name of this OperatorSummary.
        :type: str
        """
        self._name = name

    @property
    def display_name(self):
        """
        **[Required]** Gets the display_name of this OperatorSummary.
        display name of the operand


        :return: The display_name of this OperatorSummary.
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """
        Sets the display_name of this OperatorSummary.
        display name of the operand


        :param display_name: The display_name of this OperatorSummary.
        :type: str
        """
        self._display_name = display_name

    @property
    def datatype(self):
        """
        **[Required]** Gets the datatype of this OperatorSummary.
        data type of operand


        :return: The datatype of this OperatorSummary.
        :rtype: str
        """
        return self._datatype

    @datatype.setter
    def datatype(self, datatype):
        """
        Sets the datatype of this OperatorSummary.
        data type of operand


        :param datatype: The datatype of this OperatorSummary.
        :type: str
        """
        self._datatype = datatype

    @property
    def managed_listtype(self):
        """
        **[Required]** Gets the managed_listtype of this OperatorSummary.
        operand list type


        :return: The managed_listtype of this OperatorSummary.
        :rtype: str
        """
        return self._managed_listtype

    @managed_listtype.setter
    def managed_listtype(self, managed_listtype):
        """
        Sets the managed_listtype of this OperatorSummary.
        operand list type


        :param managed_listtype: The managed_listtype of this OperatorSummary.
        :type: str
        """
        self._managed_listtype = managed_listtype

    @property
    def filter_type(self):
        """
        **[Required]** Gets the filter_type of this OperatorSummary.
        Filter type can be config filter or condition filter

        Allowed values for this property are: "CONDITION", "CONFIG", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The filter_type of this OperatorSummary.
        :rtype: str
        """
        return self._filter_type

    @filter_type.setter
    def filter_type(self, filter_type):
        """
        Sets the filter_type of this OperatorSummary.
        Filter type can be config filter or condition filter


        :param filter_type: The filter_type of this OperatorSummary.
        :type: str
        """
        allowed_values = ["CONDITION", "CONFIG"]
        if not value_allowed_none_or_none_sentinel(filter_type, allowed_values):
            filter_type = 'UNKNOWN_ENUM_VALUE'
        self._filter_type = filter_type

    @property
    def operators(self):
        """
        **[Required]** Gets the operators of this OperatorSummary.
        List of parameters


        :return: The operators of this OperatorSummary.
        :rtype: list[oci.cloud_guard.models.ConditionOperator]
        """
        return self._operators

    @operators.setter
    def operators(self, operators):
        """
        Sets the operators of this OperatorSummary.
        List of parameters


        :param operators: The operators of this OperatorSummary.
        :type: list[oci.cloud_guard.models.ConditionOperator]
        """
        self._operators = operators

    @property
    def multi_list_types(self):
        """
        Gets the multi_list_types of this OperatorSummary.
        configuration value type list for multilist data type


        :return: The multi_list_types of this OperatorSummary.
        :rtype: list[str]
        """
        return self._multi_list_types

    @multi_list_types.setter
    def multi_list_types(self, multi_list_types):
        """
        Sets the multi_list_types of this OperatorSummary.
        configuration value type list for multilist data type


        :param multi_list_types: The multi_list_types of this OperatorSummary.
        :type: list[str]
        """
        self._multi_list_types = multi_list_types

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
